@@ -1,9 +1,7 @@
 import json
-from importlib.metadata import version as imp_version
 from typing import IO, Any
 
 import click
-from packaging.version import Version
 
 import black
 
@@ -39,13 +37,9 @@ def generate_schema_from_click(
 
         result[name]["description"] = param.help
 
-        if param.default is not None and not param.multiple:
-            if Version(imp_version("click")) >= Version("8.3.0"):
-                # Ignore the attr-defined error from running with click < 8.3.0, and
-                # also ignore the error for unused-ignore with click >= 8.3.0
-                if param.default is click._utils.UNSET:  # type: ignore[attr-defined, unused-ignore]
-                    continue
-            result[name]["default"] = param.default
+        default = param.to_info_dict()["default"]
+        if default is not None and not param.multiple:
+            result[name]["default"] = default
 
     return result
 
